package ru.sicamp.sicamphelper

import com.fasterxml.jackson.databind.ObjectMapper
import com.fasterxml.jackson.module.kotlin.KotlinFeature
import com.fasterxml.jackson.module.kotlin.KotlinModule
import org.jetbrains.exposed.sql.DatabaseConfig
import org.postgresql.PGProperty
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import javax.sql.DataSource
import org.postgresql.ds.PGSimpleDataSource
import org.springframework.beans.factory.annotation.Value

@Configuration
class ApplicationConfig {
    @Bean
    fun datasource(
        @Value("\${spring.datasource.url}")
        url: String,
        @Value("\${spring.datasource.username}")
        username: String,
        @Value("\${spring.datasource.password}")
        password: String,
    ): DataSource {
        val dataSource = PGSimpleDataSource()
        dataSource.setURL(url)
        dataSource.user = username
        dataSource.password = password
        dataSource.setProperty(PGProperty.REWRITE_BATCHED_INSERTS, "true")

        return dataSource
    }

    @Bean
    fun objectMapper(): ObjectMapper = ObjectMapper().registerModule(
        KotlinModule.Builder()
            .withReflectionCacheSize(512)
            .configure(KotlinFeature.NullToEmptyCollection, true)
            .configure(KotlinFeature.NullToEmptyMap, true)
            .configure(KotlinFeature.NullIsSameAsDefault, true)
            .configure(KotlinFeature.SingletonSupport, false)
            .configure(KotlinFeature.StrictNullChecks, true)
            .build()
    )

    @Bean
    fun databaseConfig(): DatabaseConfig = DatabaseConfig {
        keepLoadedReferencesOutOfTransaction = true
    }
}