package ru.sicamp.sicamphelper.api.command.impl

import mu.KLogging
import org.springframework.stereotype.Controller
import ru.sicamp.sicamphelper.api.command.Commands
import ru.sicamp.sicamphelper.api.command.PublicCommand
import ru.sicamp.sicamphelper.api.command.getValueOrNull
import ru.sicamp.sicamphelper.api.command.toLocalDate
import ru.sicamp.sicamphelper.model.request.RequestWrapper
import ru.sicamp.sicamphelper.model.data.UserInput
import ru.sicamp.sicamphelper.model.request.schedule.ScheduleRequest
import ru.sicamp.sicamphelper.model.response.ResponseWrapper
import ru.sicamp.sicamphelper.model.response.schedule.ScheduleResponse
import ru.sicamp.sicamphelper.service.ScheduleService
import ru.sicamp.sicamphelper.service.UserService

@Controller
class ScheduleCommand(
    override val userService: UserService,
    private val scheduleService: ScheduleService
) : PublicCommand<ScheduleRequest, ScheduleResponse>(Commands.SCHEDULE) {
    override suspend fun executeRequest(
        request: RequestWrapper<ScheduleRequest>
    ): ResponseWrapper<ScheduleResponse> = scheduleService.getUserSchedule(request)

    override fun extract(
        arguments: Array<out String>?
    ) = ScheduleRequest(
        date = arguments?.getValueOrNull(0).toLocalDate(),
        userInput = arguments?.getValueOrNull(1)?.let { UserInput(it) }
    )


    companion object : KLogging()
}
