package ru.sicamp.sicamphelper.api.command.impl

import org.springframework.stereotype.Component
import ru.sicamp.sicamphelper.api.command.Commands
import ru.sicamp.sicamphelper.api.command.PublicCommand
import ru.sicamp.sicamphelper.api.command.getValueOrNull
import ru.sicamp.sicamphelper.api.command.toLocalDate
import ru.sicamp.sicamphelper.model.data.GroupInput
import ru.sicamp.sicamphelper.model.request.RequestWrapper
import ru.sicamp.sicamphelper.model.request.schedule.ScheduleGroupRequest
import ru.sicamp.sicamphelper.model.response.ResponseWrapper
import ru.sicamp.sicamphelper.model.response.schedule.ScheduleResponse
import ru.sicamp.sicamphelper.service.ScheduleService
import ru.sicamp.sicamphelper.service.UserService

@Component
class ScheduleGroupCommand(
    override val userService: UserService,
    private val scheduleService: ScheduleService,
) : PublicCommand<ScheduleGroupRequest, ScheduleResponse>(Commands.SCHEDULE_GROUP) {
    override suspend fun executeRequest(request: RequestWrapper<ScheduleGroupRequest>): ResponseWrapper<ScheduleResponse> {
        return scheduleService.getSchedulesByGroup(request)
    }

    override fun extract(arguments: Array<out String>?): ScheduleGroupRequest =
        ScheduleGroupRequest(
            date = arguments?.getValueOrNull(0).toLocalDate(),
            group = arguments?.getValueOrNull(1)?.let { GroupInput(it) }
        )
}