package ru.sicamp.sicamphelper.db.entity

import org.jetbrains.exposed.dao.LongEntity
import org.jetbrains.exposed.dao.LongEntityClass
import org.jetbrains.exposed.dao.id.EntityID
import org.springframework.security.core.GrantedAuthority
import org.springframework.security.core.userdetails.UserDetails
import ru.sicamp.sicamphelper.db.table.Users
import java.time.LocalDateTime

class User(id: EntityID<Long>) : LongEntity(id), UserDetails {
    companion object : LongEntityClass<User>(Users)

    var login by Users.login
    var name by Users.name
    var supervisor by User optionalReferencedOn Users.supervisor
    var _password by Users.password
    var tgId by Users.tgId
    var tgUsername by Users.tgUsername
    var role by Users.role
    var group by Group referencedOn Users.group
    var sex by Users.sex
    var age by Users.age
    var locked by Users.locked
    var enabled by Users.enabled
    var accountExpiration by Users.accountExpiration
    var credentialsExpiration by Users.credentialsExpiration

    override fun getAuthorities(): MutableCollection<out GrantedAuthority> = Users.Role.lowerRoles(role).map {
        GrantedAuthority { role.name }
    }.toMutableList()

    override fun getPassword(): String = _password

    override fun getUsername(): String = login

    override fun isAccountNonExpired(): Boolean = LocalDateTime.now() < accountExpiration

    override fun isAccountNonLocked(): Boolean = !locked

    override fun isCredentialsNonExpired(): Boolean = LocalDateTime.now() < credentialsExpiration

    override fun isEnabled(): Boolean = enabled

    override fun toString(): String {
        return "User: {login:$login, tgUsername:$tgUsername, tgId:$tgId, role:$role}"
    }
}
