package ru.sicamp.sicamphelper.model.data

/**Class extracts user from user input. Also used to represent user in views. Format: @tgUsername|$gateLogin|#userId */
class UserInput(
    input: String
) {
    val type: Type? = Type.typeByPrefix(input.firstOrNull())

    val tgUsername = if (type == Type.TG_USERNAME) input.tail().toString() else null
    val gateLogin = if (type == Type.GATE_LOGIN) input.tail().toString() else null
    val userId = if (type == Type.USER_ID) input.tail().toString().toLong() else null

    enum class Type(val prefix: Char) {
        TG_USERNAME('@'), GATE_LOGIN('$'), USER_ID('#');

        companion object {
            fun typeByPrefix(char: Char?) = values().find { it.prefix == char }
        }
    }

    companion object {
        fun fromTgUser(tgUser: org.telegram.telegrambots.meta.api.objects.User) = UserInput(Type.TG_USERNAME.prefix + tgUser.userName)
    }
}