package ru.sicamp.sicamphelper.model.response

import org.telegram.telegrambots.meta.api.methods.send.SendMessage
import org.telegram.telegrambots.meta.api.objects.MessageEntity
import org.telegram.telegrambots.meta.api.objects.replykeyboard.ReplyKeyboard
import ru.sicamp.sicamphelper.util.ifNotEmpty
import ru.sicamp.sicamphelper.util.ifNotNull

abstract class TelegramResponse : Response {
    protected abstract val text: String
    protected open val parseMode: String? = null
    protected open val disableWebPagePreview: Boolean? = null
    protected open val disableNotification: Boolean? = null
    protected open val replyToMessageId: Int? = null
    protected open val replyMarkup: ReplyKeyboard? = null
    protected open val entities: List<MessageEntity> = emptyList()
    protected open val allowSendingWithoutReply: Boolean? = null
    protected open val protectContent: Boolean = true

    fun buildMessage() = SendMessage.builder()
        .text(text)
        .ifNotNull(parseMode) { parseMode(parseMode) }
        .ifNotNull(disableWebPagePreview) { disableWebPagePreview(disableWebPagePreview) }
        .ifNotNull(disableNotification) { disableNotification(disableNotification) }
        .ifNotNull(replyToMessageId) { replyToMessageId(replyToMessageId) }
        .ifNotNull(replyMarkup) { replyMarkup(replyMarkup) }
        .ifNotEmpty(entities) { entities(entities) }
        .ifNotNull(allowSendingWithoutReply) { allowSendingWithoutReply(allowSendingWithoutReply) }
        .protectContent(protectContent)!!
}