package ru.sicamp.sicamphelper.service

import org.jetbrains.exposed.sql.and
import org.jetbrains.exposed.sql.transactions.transaction
import org.springframework.security.core.userdetails.UserDetailsService
import org.springframework.security.crypto.password.PasswordEncoder
import org.springframework.stereotype.Service
import ru.sicamp.sicamphelper.api.controller.UsersController
import ru.sicamp.sicamphelper.db.entity.User
import ru.sicamp.sicamphelper.db.table.Users
import ru.sicamp.sicamphelper.model.dto.UserDto
import java.time.Duration
import java.time.LocalDateTime

@Service
class UserService(
    private val passwordEncoder: PasswordEncoder
) : UserDetailsService {
    private val userExpirationTime = Duration.ofDays(60)
    private val credentialsExpirationTime = Duration.ofDays(30)
    
    fun findUserByTgUser(user: org.telegram.telegrambots.meta.api.objects.User): User? = transaction {
        User.find {
            (Users.tgId eq user.id) and
                    (Users.tgUsername eq user.userName)
        }.firstOrNull()
    }

    override fun loadUserByUsername(username: String?): User = transaction {
        username?.let {
            User.find { Users.login eq it }.firstOrNull()
        } ?: error("Not found user with username $username")
    }
    
    fun createUser(userDto: UserDto): User {
        val now = LocalDateTime.now()
        UsersController.logger.info("Creating new user at $now")
        return User.new {
            login = userDto.login
            _password = passwordEncoder.encode(userDto.password)
            role = userDto.role
            name = userDto.name
            sex = userDto.sex
            age = userDto.age
            tgId = userDto.tgId
            tgUsername = userDto.tgUsername
            locked = false
            enabled = userDto.enabled
            accountExpiration = now + userExpirationTime
            credentialsExpiration = now + credentialsExpirationTime
        }
    }
}